#!/usr/bin/perl -w
#
# Copyright (C) 2008 Patrick Fuerlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#//////////////////////////////////////////////////////////////////////////////
BEGIN {
  push( @INC, "/opt/rbc/lib/" );
  push( @INC, "../lib/" );
}

use strict;
use warnings;
use ReelBox::Base qw(vdr_status get_config);
use Storable;
use File::stat;
use Getopt::Long;
use LockFile::Simple qw(lock trylock unlock);
use File::Find;
use File::Basename;
use URI::Escape;
use Time::Local;

#//////////////////////////////////////////////////////////////////////////////
my $g_IP            = '192.168.0.176';
my $href            = get_config('/opt/rbc/htdocs/config/reelbox.cfg');
my %g_RbcConf       = %{$href};
my $g_UpdateFlag    = "/media/reel/recordings/.update";
my $g_RbcStatusHash = "/opt/rbc/var/rbc.hash";
my $g_RbcRecordHash = "/opt/rbc/var/recordings.hash";
my $g_RecDir        = $g_RbcConf{'RecDir'} || '/media/reel/recordings/';
my %g_RbcStatus;

my $g_Debug;
my %g_Config;

## init signal exit_handler
$SIG{'KILL'} = \&exit_handler;
$SIG{'INT'}  = \&exit_handler;
$SIG{'TERM'} = \&exit_handler;

## buffer off
$| = 1;
select(*STDERR);
$| = 1;
select(*STDOUT);

#//////////////////////////////////////////////////////////////////////////////
# function forward declaration
sub GetRecords;
sub GetRecordDetail;
sub CheckUpdateFlag;
sub GetArgs;
sub exit_handler;
sub main;

#//////////////////////////////////////////////////////////////////////////////
# funtions
sub GetArgs {
  my $help;

  GetOptions(
    "--debug"      => \$g_Debug,
    "--force-load" => \$g_Config{'force-load'},
    "--help"       => \$help
  );

}

#//////////////////////////////////////////////////////////////////////////////
sub GetRecords {
  my $new_update_time = shift;
  my %NewRec;
  my %Records;
  my @OUT;

  ## get rbc status
  if ( -e $g_RbcRecordHash ) {
    my $href;
    if ( eval { $href = retrieve( $g_RbcRecordHash ) } ) {
      print "load record hash ok\n" if $g_Debug;
      %Records = %{$href};
    }
    else {
      unlink($g_RbcRecordHash);
      print "load record hash failed! Unlink it ...\n" if $g_Debug;
    }
  }

  my $RecID = 0;
  open( FIND, "find $g_RecDir -follow -name \"info.vdr\" |grep \".rec/\" |" )
    or die "";
  while (<FIND>) {
    chomp;
    $RecID++;
    my $key = $_;
    (my $path = $_) =~ s/$g_RecDir//;

    print "REC: $_\n" if $g_Debug;
    my $href = GetRecordDetail($_);

    ## get path info
    my $cntr = 0;
    foreach ( split( /\//, $path ) ) {
      my $dir = undef;
      $cntr++;

      $dir = 1 if( /\.rec$/ );

      next if ( /^info.vdr$/);
      $NewRec{$key}{'first'} = "false";

      if($cntr == 1) {
        $NewRec{$key}{'path'} = $path;
        $NewRec{$key}{'apath'}[$cntr] = $_ . "/";
      }
      elsif($cntr == 2) {
        $NewRec{$key}{'apath'}[$cntr] =
          $NewRec{$key}{'apath'}[$cntr-1] . $_ . "/";

        $NewRec{$key}{'first'}        = "true" if defined $dir;

        $NewRec{$key}{'recpath'}      =
          $NewRec{$key}{'apath'}[$cntr] if defined $dir;
      }
      else {
        $NewRec{$key}{'apath'}[$cntr] =
          $NewRec{$key}{'apath'}[$cntr-1] . $_ . "/";
        $NewRec{$key}{'recpath'} = $NewRec{$key}{'apath'}[$cntr-1] if defined $dir;
      }
    }

    ## check details
    if ( ref($href) ne "HASH" ) {
      print STDERR "error get detail[$RecID]: $href\n" if $g_Debug;
      $NewRec{$key}{'status'}      = 'error_get_detail';
      $NewRec{$key}{'status_text'} = $href;
      next;
    }

    ## save records
    $NewRec{$key}{detail}            = \%{$href};
    $NewRec{$key}{'detail'}{'RecID'} = $RecID;
    $NewRec{$key}{'RecID'}           = $RecID;
    $NewRec{$key}{'titel'}           = $NewRec{$key}{'detail'}{'titel'};
    $NewRec{$key}{'esc_titel'}       = $NewRec{$key}{'detail'}{'esc_titel'};
    $NewRec{$key}{'status'}          = 'loaded';
    $NewRec{$key}{'recdir'}          = dirname($key);

    ## check resume status
    if ( -e "$NewRec{$key}{'recdir'}/resume.vdr" ) {
      $NewRec{$key}{'resume'} = 1;
      $NewRec{$key}{'detail'}{'resume'} = 1;
    }
    else {
      $NewRec{$key}{'resume'} = undef;
      $NewRec{$key}{'detail'}{'resume'} = undef;
    }

    store \%NewRec, $g_RbcRecordHash;
  }
  close(FIND);

  ## save RbStatus
  store \%NewRec, $g_RbcRecordHash;

  return 'done';
}

#//////////////////////////////////////////////////////////////////////////////
sub GetRecordDetail {
  my $RecDir = shift;
  my %Detail;

  ## init structure
  $Detail{'audio'} = [];

  open( VDR, "< $RecDir" ) or die "";
  while (<VDR>) {
    chomp;
    s/^215-//;

    $Detail{'channel'}   = $1 if (/^C\s(.+\s*)$/);
    $Detail{'timestamp'} = $1 if (/^E\s(.+\s*)$/);
    $Detail{'titel'}     = $1 if (/^T\s(.+\s*)$/);
    $Detail{'subtitel'}  = $1 if (/^S\s(.+\s*)$/);
    $Detail{'descr'}     = $1 if (/^D\s(.+\s*)$/);
    $Detail{'vps'}       = $1 if (/^V\s(.+\s*)$/);
    $Detail{'epg'}       = $1 if (/^@\s(.+\s*)$/);
    $Detail{'hd'}        = 'true' if (/^X\s(.+HD-Video\s*)$/);

    push(@{$Detail{'audio'}} , $1) if (/^X\s(.+\s*)$/);

    $Detail{'esc_titel'} = uri_escape( $Detail{'titel'} );

    ## set time format
    if ( defined $Detail{'timestamp'} ) {
      if ( $Detail{'timestamp'} =~ /^(\d+) (\d{10}) (\d+?) (.*)$/ ) {
        my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
          localtime($2);

        $sec  = sprintf( "%02d", $sec );
        $min  = sprintf( "%02d", $min );
        $mon  = sprintf( "%02d", $mon + 1 );
        $mday = sprintf( "%02d", $mday );

        $Detail{'time'} = "$mday.$mon." . ( int($year) + 1900 ) . " $hour:$min";

        $hour = sprintf( "%02d", $hour );
        $Detail{'timeindex'} =
          ( int($year) + 1900 ) . $mon . $mday . $hour . $min . $sec;

        ## record length
        $Detail{'length'} = int( $3 / 60 ) if defined $3;
        print "set length: $Detail{'length'} \n" if $g_Debug;

      }
    }

    ## <channel>23 - kabel eins</channel>
    if ( defined $Detail{'epg'} ) {
      ( $Detail{'channel_name'} = $Detail{'epg'} ) =~
        s/^.*<channel>(.*)<\/channel>.*$/$1/;
    }

    print "DETAIL: $_\n" if $g_Debug;

  }
  close(VDR);

  return \%Detail;
}

#//////////////////////////////////////////////////////////////////////////////
sub exit_handler {
  my $signal = shift;    # signal-nummer besorgen

  ## try to unlock
  # unlock($g_LockFile);

  unlock('/tmp/rbc');

  exit 1;
}

#//////////////////////////////////////////////////////////////////////////////
sub CheckUpdateFlag {
  my $last_update = shift || 0;

  # hack
  my $file  = `ls -1tr $g_RecDir|tail -1`;
  my $count = `find $g_RecDir/ -name info.vdr|grep ".rec/"| wc -l`;
  return if ( !defined $file );
  chomp($file);
  chomp($count);

  my $st = stat("$g_RecDir/$file") or die "Couldn't stat $g_RecDir/$file: $!";
  my $mtime = $st->mtime;

  $mtime .= $count;

  print "SET TIME $mtime\n" if $g_Debug;

  if ( $last_update == $mtime ) {
    return undef;
  }
  else {
    print "Time: $mtime\n";
  }

  return $mtime;

}

#//////////// //////////////////////////////////////////////////////////////////
# program starts here
sub main {

  ## init
  GetArgs;

  ##  check vdrstatus
  # if(! vdr_status && ! $g_Config{'force-load'}) {
  #   print "ReelVDR not running!\n" if $g_Debug;
  #  exit 0;
  # }

  ## lock
  #if(! lock('/tmp/rbc')) {
  #  print "cat't lock!\n";
  #  exit 1;
  #}

  ## get rbc status
  if ( -e $g_RbcStatusHash ) {
    my $href = retrieve($g_RbcStatusHash);
    if ( ref($href) eq "HASH" ) {
      %g_RbcStatus = %{$href};
    }
    else {
      unlink($g_RbcStatusHash);
    }
    ## force-load
    $g_RbcStatus{'last_update'} = 1 if ( defined $g_Config{'force-load'} );

  }
  else {
    $g_RbcStatus{'last_update'} = 1;
  }

  my $new_update_time = CheckUpdateFlag;

  ## read record overview
  if ( GetRecords($new_update_time) eq 'done' ) {
    ## save new update timestamp
    $g_RbcStatus{'last_update'} = $new_update_time;
  }

  ## save RbStatus
  store \%g_RbcStatus, $g_RbcStatusHash;

  ## try unlock
  #unlock('/tmp/rbc');

  exit 0;
}
main;
print STDERR "unexspected end of script\n";
exit_handler;

#//////////////////////////////////////////////////////////////////////////////
END {
  ## unlock
}
